/***********************************************/
/*                                             */
/* 10 Blinker-, 7 Servo-,                      */
/* 1 Performanceprozess,                       */
/* 1 IR-Detektor fr 4 Sensoren                */
/* Einige Prozesse erzeugen Ausgaben auf dem   */
/* LCD-Display, welche durch eine Semaphore    */
/* gekapselt werden mssen. Der damit erzeugte */
/* Engpass verlangsamt das System.             */
/*                                             */
/* Die Prozesse werden nacheinander gestartet, */
/* an der Ausgabe des Performanceprozesses ist */
/* seine abnehmende Leistung zu erkennen       */
/*                                             */
/* 01.02.05                                    */
/* I. Boersch                                  */
/*                                             */
/***********************************************/

#include <stdio.h>
#include <regc515c.h>
#include <stub.h>
#include "sema.h"

#define WAIT_BEFORE_START 1000
#define HALF_PERIOD 500

#define LCD_MUTEX 1                               // Semaphore fuer LCD

void set_servo(unsigned char s, unsigned int ms)
{
    if ( s < 3 )        servo(s,ms);
    else if ( s < 11 )  servobank(s-3,ms);
}


// Prozessklasse Servo
void servobank_test( unsigned char servo, unsigned int zeit)
{
    unsigned int servozeit;
    servozeit=zeit;

    do
    {
        sleep(500);

        servozeit+=10;
        if (servozeit>2100) servozeit=600;

        set_servo(servo,servozeit);
    }
    while(1);
}


// Prozessobject Servo
void servo0_test() { servobank_test(0,1000);}
void servo1_test() { servobank_test(1,1010);}
void servo2_test() { servobank_test(2,1020);}
void servo3_test() { servobank_test(3,1030);}
void servo4_test() { servobank_test(4,1040);}
void servo5_test() { servobank_test(5,1050);}
void servo6_test() { servobank_test(6,1060);}
void servo7_test() { servobank_test(7,1070);}

// Prozessklasse Blink
void blink(unsigned char id)
{

    while(1)
    {

        S_down(LCD_MUTEX);
        lcd_setxy(0,id);
        lcd_putchar('_');
        S_up(LCD_MUTEX);
        if (id < 4 ) led(id,0);

        sleep(HALF_PERIOD);

        if (id < 4 ) led(id,1);
        S_down(LCD_MUTEX);
        lcd_setxy(0,id);
        lcd_putchar('A'+id);
        S_up(LCD_MUTEX);

        sleep(HALF_PERIOD);
    }
}


// Prozessobject Blink
void p0( void) {blink(0);}
void p1( void) {blink(1);}
void p2( void) {blink(2);}
void p3( void) {blink(3);}
void p4( void) {blink(4);}
void p5( void) {blink(5);}
void p6( void) {blink(6);}
void p7( void) {blink(7);}
void p8( void) {blink(8);}
void p9( void) {blink(9);}
void p10( void) {blink(10);}

// Detektor fr 125 Hz an D4-D7
void ir_proc4(void)
{
#define IR_PROC_HALF_PERIOD 4                 /* 125 Hz, Periode damit 8 ms */
    /* maximal 10 Perioden abwarten */
#define IR_PROC_WAIT (10 * 2 * IR_PROC_HALF_PERIOD)
#define IR_PROC_PERIODS_FOR_OK 3              /* notw. Anzahl fr Detektion */
#define IR_PROC_MAX_ERROR_PER_PERIOD 3        /* erlaubte Fehlertakte pro Periode */

    unsigned char empfang[4];
    unsigned long endezeit;
    unsigned char i;

    mod_ir0_maxfehler(IR_PROC_MAX_ERROR_PER_PERIOD);
    mod_ir1_maxfehler(IR_PROC_MAX_ERROR_PER_PERIOD);
    mod_ir2_maxfehler(IR_PROC_MAX_ERROR_PER_PERIOD);
    mod_ir3_maxfehler(IR_PROC_MAX_ERROR_PER_PERIOD);

    mod_ir_aus();                                 // Geschwindigkeits-Anzeige
    motor_pwm(0,10);                              // Empfangsanzeige
    motor_pwm(1,10);
    motor_pwm(2,10);
    motor_pwm(3,10);

    do
    {
        mod_ir_empf0_gutzaehler = 0;              /* mod_irX_status auf 0 */
        mod_ir_empf1_gutzaehler = 0;              /* mod_irX_status auf 0 */
        mod_ir_empf2_gutzaehler = 0;              /* mod_irX_status auf 0 */
        mod_ir_empf3_gutzaehler = 0;              /* mod_irX_status auf 0 */

        mod_ir0_takt(IR_PROC_HALF_PERIOD);        /* Detektor an */
        mod_ir1_takt(IR_PROC_HALF_PERIOD);        /* Detektor an */
        mod_ir2_takt(IR_PROC_HALF_PERIOD);        /* Detektor an */
        mod_ir3_takt(IR_PROC_HALF_PERIOD);        /* Detektor an */

        // exclusive busy waiting
        mod_ir_an();                              // Geschwindigkeits-Anzeige
        endezeit = akt_time() + IR_PROC_WAIT;
        process_hog();

        // Messung
        while (endezeit > akt_time())
        {
            empfang[0] = mod_ir0_status();
            empfang[1] = mod_ir1_status();
            empfang[2] = mod_ir2_status();
            empfang[3] = mod_ir3_status();

            if (empfang[0] >= IR_PROC_PERIODS_FOR_OK &&
                empfang[1] >= IR_PROC_PERIODS_FOR_OK &&
                empfang[2] >= IR_PROC_PERIODS_FOR_OK &&
                empfang[3] >= IR_PROC_PERIODS_FOR_OK ) break;

        }
        process_defer();
        mod_ir0_takt(0);                          /* Detektor aus */
        mod_ir1_takt(0);                          /* Detektor aus */
        mod_ir2_takt(0);                          /* Detektor aus */
        mod_ir3_takt(0);                          /* Detektor aus */

        //Anzeige mit LCD
        S_down(LCD_MUTEX);
        lcd_setxy(1,0);
        for ( i = 0; i < 4; i++)
        {
            lcd_ubyte( empfang[i] );
            if (empfang[i] < 10) lcd_putchar(' ');
            lcd_putchar(' ');
        }
        S_up(LCD_MUTEX);

        // Anzeige mit LED
        for ( i = 0; i < 4; i++)
        {

            if (empfang[i] >= IR_PROC_PERIODS_FOR_OK )  {motor_richtung(i,1);}
            else { motor_richtung(i,0); }
        }

        mod_ir_aus();                             // Geschwindigkeits-Anzeige
        sleep(20);                                /* enthlt process_defer() */

        // Ausgabe in globale Variable
        // ...
    }
    while(1);
}


void perf_checker()
{
    float x,y,perf;
    unsigned long endezeit;
#define MESS_ZEIT 500
    do
    {
        endezeit = akt_time() + MESS_ZEIT;

        perf = 0.0;
        x = 42.42;

        while (endezeit > akt_time())
        {
            y = x * x * x;
            perf++;
        }

        S_down(LCD_MUTEX);
        lcd_setxy(0,12);
        lcd_uint((unsigned int)perf * 2);
        if (perf < 1000) lcd_putchar(' ');
        if (perf < 100 ) lcd_putchar(' ');
        if (perf < 10  ) lcd_putchar(' ');
        S_up(LCD_MUTEX);

    } while(1);

}


/************************************************/
/* MAIN-Routine									*/
/************************************************/
void AksenMain(void)
{
    S_init(LCD_MUTEX,1);

    /* Starten der Prozesse */
    process_start( perf_checker, 10); sleep(WAIT_BEFORE_START);

    process_start( ir_proc4, 10); sleep(10000);

    process_start( servo0_test, 150);
    process_start( servo1_test, 160);
    process_start( servo2_test, 170);
    process_start( servo3_test, 180);
    process_start( servo4_test, 190);
    process_start( servo5_test, 200);
    process_start( servo6_test, 210);
    servobank_start();

    sleep(5000);

    process_start( p0, 10); sleep(WAIT_BEFORE_START);
    process_start( p1, 10); sleep(WAIT_BEFORE_START);
    process_start( p2, 10); sleep(WAIT_BEFORE_START);
    process_start( p3, 10); sleep(WAIT_BEFORE_START);
    process_start( p4, 10); sleep(WAIT_BEFORE_START);
    process_start( p5, 10); sleep(WAIT_BEFORE_START);
    process_start( p6, 10); sleep(WAIT_BEFORE_START);
    process_start( p7, 10); sleep(WAIT_BEFORE_START);
    process_start( p8, 10); sleep(WAIT_BEFORE_START);
    process_start( p9, 10); sleep(WAIT_BEFORE_START);

    p10();
}
