/***********************************************/
/*                                             */
/* CAN-Routinen                                */
/*                                             */
/* 18.07.2003                                  */
/* K.-U. Mrkor                                 */
/*                                             */
/***********************************************/

#include <stdio.h>
#include <regc515c.h>
#include <stub.h>

#include "can.h"

/****************************************/
/* Initialisierung des SJA1000			*/
/****************************************/
void CanInit( void)
{
    /*** Alle Interrupts sperren ***/
    EA=0;

    /*** SJA100 in den RESET-Modus bringen ***/
    while( (CAN_MODE & 1)==0x00)
    {
        CAN_MODE &=1;                             /* ResetRequest=1 */
    }

    /*** ClockDivider setzen ***/
    CAN_CLOCK_DIVIDER=0x80 | 0x40 | 0x08;         /* 0x80 Pelican-Modus */
    /* 0x40 bypass CAN input-comperator */
    /* 0x08 ClkOut-Pin deaktivieren  */

    /*** Alle CAN-Interrupts sperren ***/
    //CAN_INTERRUPT_ENABLE=0x00;

    /** Akzeptanz-Filter setzen  **/
    CAN_ACCEPTANCE_MASKE0=0xFF;
    CAN_ACCEPTANCE_MASKE1=0xFF;
    CAN_ACCEPTANCE_MASKE2=0xFF;
    CAN_ACCEPTANCE_MASKE3=0xFF;

    CAN_ACCEPTANCE_CODE0=0x00;
    CAN_ACCEPTANCE_CODE1=0x00;
    CAN_ACCEPTANCE_CODE2=0x00;
    CAN_ACCEPTANCE_CODE3=0x00;

    /*** BUS-Timing konfigurieren ***/
    /***********************/
    /* Baudrate BTR0 BTR 1 */
    /*---------------------*/
    /* 100k 	4E		14 */
    /* 125k		C5		3A */
    /* 250k		C2		49 */
    /* 500k		41		27 */
    /* 1M		40		27 */
    /***********************/
    CAN_BUS_TIMING0=0xC5;
    CAN_BUS_TIMING1=0x3A;

    //CAN_INTERRUPT_ENABLE=0xFF;

    /*** OutputControl ***/
    CAN_OUTPUT_CONTROL=0x1A;

    /*** in OperationsMode schalten ***/
    do
    {
        CAN_MODE=0x00;
    }
    while((CAN_MODE & 1) != 0x00);

    /*** allgemeine Interruptfreigabe ***/
    EA=1;
}


/****************************************/
/* MINI-Version eines CAN-Empfaengers	*/
/****************************************/
int CanEmpfang( struct CAN_MSG xdata *CanBotschaft)
{
    unsigned char status;

    status=CAN_STATUS;                            /* Empfangsstatus abfragen */
    if (!(status & 0x01))
    {
        return FALSE;
    }

    /* Nachricht in Puffer kopieren */
    CanBotschaft->Puffer[0]=CAN_RECEIVE_1;
    CanBotschaft->Puffer[1]=CAN_RECEIVE_2;
    CanBotschaft->Puffer[2]=CAN_RECEIVE_3;
    CanBotschaft->Puffer[3]=CAN_RECEIVE_4;
    CanBotschaft->Puffer[4]=CAN_RECEIVE_5;
    CanBotschaft->Puffer[5]=CAN_RECEIVE_6;
    CanBotschaft->Puffer[6]=CAN_RECEIVE_7;
    CanBotschaft->Puffer[7]=CAN_RECEIVE_8;

    /* Nachrichten-ID auslesen */
    CanBotschaft->ID[0]=CAN_RECEIVE_ID1;
    CanBotschaft->ID[1]=CAN_RECEIVE_ID2;
    CanBotschaft->ID[2]=CAN_RECEIVE_ID3;
    CanBotschaft->ID[3]=CAN_RECEIVE_ID4;

    /* Nachrichten-Laenge auslesen */
    CanBotschaft->DatenLaenge=CAN_RECEIVE_FRAMEINFO & 0x0F;

    /* Empfangspuffer wieder freigeben */
    CAN_COMMAND = 0x04;

    return TRUE;

}


/****************************************/
/* MINI-Version eines CAN-Senders		*/
/****************************************/
int CanSenden(  struct CAN_MSG  xdata *CanBotschaft)
{
    /* Frame-Info (hwt: Extended Frame  -  nwt: Anzahl Datenbytes) */
    CAN_TRANSMIT_FRAMEINFO=0x80 | (CanBotschaft->DatenLaenge & 0x0F);

    /* Nachrichten-ID im Sendepuffer ablegen */
    CAN_TRANSMIT_ID1=CanBotschaft->ID[0];
    CAN_TRANSMIT_ID2=CanBotschaft->ID[1];
    CAN_TRANSMIT_ID3=CanBotschaft->ID[2];
    CAN_TRANSMIT_ID4=CanBotschaft->ID[3];

    /* Nachricht im Sendepuffer ablegen */
    CAN_TRANSMIT_1=CanBotschaft->Puffer[0];
    CAN_TRANSMIT_2=CanBotschaft->Puffer[1];
    CAN_TRANSMIT_3=CanBotschaft->Puffer[2];
    CAN_TRANSMIT_4=CanBotschaft->Puffer[3];
    CAN_TRANSMIT_5=CanBotschaft->Puffer[4];
    CAN_TRANSMIT_6=CanBotschaft->Puffer[5];
    CAN_TRANSMIT_7=CanBotschaft->Puffer[6];
    CAN_TRANSMIT_8=CanBotschaft->Puffer[7];

    /* Senden */
    CAN_COMMAND = 0x01;

    return TRUE;

}
