/***********************************************/
/*                                             */
/* Test der Berechnung des globalen Speichers, */
/* ein Byte mehr erzeugt hier einen Fehler.    */
/* Die berschreitung des globalen Speichers   */
/* wird nicht abgefangen und erzeugt zur       */
/* Laufzeit Programmfehler, die meist schwer   */
/* zu entdecken sind.                          */
/*                                             */
/* Deshalb: Vermeiden durch Berechnen!         */
/*                                             */
/* 02.02.2005                                  */
/* I. Boersch                                  */
/*                                             */
/***********************************************/

#include <stdio.h>
#include <regc515c.h>
#include <stub.h>

// verfgbarer globaler Speicher bei 3 Prozessen:
// 7680 - 3 x 256 - 16 = 6896

// die Hlfte ist:
// 6896 / 2 = 3448
#define GLOB_MEM_HALF 3448
unsigned char daten [2][GLOB_MEM_HALF];

/*********************************/
/* Checker-Prozessklasse         */
/*********************************/

void checker(unsigned char id)
{
    unsigned int i;

    lcd_setxy(id,0);
    lcd_puts("ok");

    do
    {

        // init array
        for ( i=0; i< GLOB_MEM_HALF; i++ )
        {
            daten[id][i] = (unsigned char)(id + i);
        }

        //check array
        for ( i=0; i< GLOB_MEM_HALF; i++ )
        {
            if (daten[id][i] != (unsigned char)(id + i))
            {
                lcd_cls();
                lcd_puts("Error!");
                while(1);
            }
        }
    } while(1);
}


/*********************************/
/* Checker-Prozessobjekte        */
/*********************************/
void p0( void) {checker(0);}
void p1( void) {checker(1);}

/************************************************/
/* MAIN-Routine									*/
/************************************************/
void AksenMain(void)
{

    /* Starten der Prozesse */
    process_start( p0, 10); sleep(500);
    process_start( p1, 10);

    /* harakiri */
    process_kill( process_get_pid());
}
