/***********************************************/
/*                                             */
/* Blinker mit 16 Prozessen                    */
/* dazu Empfang von 125Hz Infrarot             */
/* 12.05.2004                                  */
/* I. Boersch                                  */
/*                                             */
/***********************************************/

/* Grundidee: Der Detektor des 125Hz-IR luft  */
/* in der Systemzeitscheibe und wird durch     */
/* Prozesswechsel gestrt, daher ist es        */
/* notwendig dem Detektor gengend lange Zeit  */
/* (ca. 5-10 IR-Signal-Perioden reichen) zu    */
/* verschaffen. Der Prozess ir_proc blockiert  */
/* hierzu den Prozesswechsel fr diese Zeit    */
/* und gibt dann mglichst schnell den         */
/* Prozessor weiter.                           */
/***********************************************/

#include <stdio.h>
#include <regc515c.h>
#include <stub.h>

#define WAIT_BEFORE_START 1000
#define HALF_PERIOD 500

// Detektor fr 125 Hz an D4
void ir_proc(void)
{
    // Frequenz
#define IR_PROC_HALF_PERIOD 4                 /* 125 Hz, Periode damit 8 ms */

                                                  /* mindestens 5-.. x Periode */
#define IR_PROC_WAIT (10 * 2 * IR_PROC_HALF_PERIOD)
#define IR_PROC_PERIODS_FOR_OK 3              /* notw. Anzahl fr Detektion */
#define IR_PROC_MAX_ERROR_PER_PERIOD 3        /* erlaubte Fehlertakte pro Periode */

    unsigned char empfang;
    unsigned long endezeit;

    mod_ir0_maxfehler(IR_PROC_MAX_ERROR_PER_PERIOD);
    motor_pwm(3,10);

    do
    {
        mod_ir0_takt(IR_PROC_HALF_PERIOD);        /* Detektor an */
        mod_ir_empf0_gutzaehler = 0;              /* mod_irX_status auf 0 */

        // exclusive busy waiting
        motor_richtung(3,1);
        endezeit = akt_time() + IR_PROC_WAIT;
        process_hog();
        while (endezeit > akt_time())
        {
            if ((empfang = mod_ir0_status()) >= IR_PROC_PERIODS_FOR_OK) break;
        }
        mod_ir0_takt(0);                          /* Detektor aus */
        motor_richtung(3,0);

        //Anzeige mit LCD
        lcd_setxy(1,0);
        lcd_ubyte( empfang );
        if (empfang < 100) lcd_putchar(' ');
        if (empfang < 10) lcd_putchar(' ');

        // Anzeige mit LED
        if (empfang >= IR_PROC_PERIODS_FOR_OK )  {mod_ir_an();}
        else { mod_ir_aus(); }

        sleep(20);                                /* enthlt process_defer() */

        // Ausgabe in globale Variable
        // ...
    }
    while(1);
}


void blink(unsigned char id)
{

    while(1)
    {
        lcd_setxy(0,id);
        lcd_ubyte(0);
        if (id < 4 ) led(id,0);

        sleep(HALF_PERIOD);

        if (id < 4 ) led(id,1);
        lcd_setxy(0,id);
        lcd_ubyte(1);

        sleep(HALF_PERIOD);
    }
}


/*********************************/
/* Blinkprozesse                 */
/*********************************/
void p0( void) {blink(0);}
void p1( void) {blink(1);}
void p2( void) {blink(2);}
void p3( void) {blink(3);}
void p4( void) {blink(4);}
void p5( void) {blink(5);}
void p6( void) {blink(6);}
void p7( void) {blink(7);}
void p8( void) {blink(8);}
void p9( void) {blink(9);}
void p10( void) {blink(10);}
void p11( void) {blink(11);}
void p12( void) {blink(12);}
void p13( void) {blink(13);}
void p14( void) {blink(14);}
void p15( void) {blink(15);}
void p16( void) {blink(16);}

/************************************************/
/* MAIN-Routine									*/
/************************************************/
void AksenMain(void)
{

    /* Starten der Prozesse */
    process_start( ir_proc, 10); sleep(10000);

    process_start( p0, 10); sleep(WAIT_BEFORE_START);
    process_start( p1, 10); sleep(WAIT_BEFORE_START);
    process_start( p2, 10); sleep(WAIT_BEFORE_START);
    process_start( p3, 10); sleep(WAIT_BEFORE_START);
    process_start( p4, 10); sleep(WAIT_BEFORE_START);
    process_start( p5, 10); sleep(WAIT_BEFORE_START);
    process_start( p6, 10); sleep(WAIT_BEFORE_START);
    process_start( p7, 10); sleep(WAIT_BEFORE_START);
    process_start( p8, 10); sleep(WAIT_BEFORE_START);
    process_start( p9, 10); sleep(WAIT_BEFORE_START);
    process_start( p10, 10); sleep(WAIT_BEFORE_START);
    process_start( p11, 10); sleep(WAIT_BEFORE_START);
    process_start( p12, 10); sleep(WAIT_BEFORE_START);
    process_start( p13, 10); sleep(WAIT_BEFORE_START);
    process_start( p14, 10); sleep(WAIT_BEFORE_START);
    process_start( p15, 10); sleep(WAIT_BEFORE_START);
    process_start( p16, 10); sleep(WAIT_BEFORE_START);

    /* harakiri */
    process_kill( process_get_pid());
}
