/***********************************************/
/* process.h                                   */
/*                                             */
/* 02.02.2004                                  */
/* K.-U. Mrkor                                 */
/***********************************************/


#ifndef __process__
#define __process__




#define PROCESS_ERROR 25	/* Allgemeiner Prozess-Fehler */





/*******************/
/* Prozess-Kontext */
/*******************/
struct prozess_kontext
{
	/* der PC muss nicht gerettet werden, da er schon auf dem Stack liegt */

	unsigned char PSW; 					/* Programm-Statuswort (Flag-Register) */

	unsigned char A;					/* Akkumulator und Hilfsakkumulator */
	unsigned char B;

	unsigned char R0;					/* Die Arbeitsregister R0-R7 */
	unsigned char R1;
	unsigned char R2;
	unsigned char R3;
	unsigned char R4;
	unsigned char R5;
	unsigned char R6;
	unsigned char R7;

	unsigned char DPL;					/* Der 16-Bit Datenzeiger DPTR */
	unsigned char DPH;

	unsigned char SP;					/* der Stack-Pointer */

	unsigned char IDATA[8];				/* IDATA */
	unsigned char STACK[223];			/* Stack */


};

/***********************/
/* Die Prozess-Tabelle */
/***********************/
struct prozess_tabelle
{
/* #(_proz_tab + 0x00) */unsigned char akt_proz;		/* welcher Prozess ist gerade aktiv */
/* #(_proz_tab + 0x01) */unsigned char ticks_aktuell;	/* wie viel Ticks hat der Prozess noch */

/* #(_proz_tab + 0x02) */unsigned char anz_proz;		/* wieviel Prozesse gibt es ueberhaupt */

/* #(_proz_tab + 0x03) */unsigned char proz_status[20]; /* 0->Slot ist leer, 1->Slot ist belegt */
/* #(_proz_tab + 0x17) */unsigned char proz_ticks[20];	/* die beim Aufruf zugewiesenen Ticks */

/* #(_proz_tab + 0x2B) */unsigned char schedule_verbot;	/* 0->Scheduling erlaubt, 1->Scheduling verboten */

/* #(_proz_tab + 0x2C) */unsigned char proz_liste[20]; 	/* Liste aller Prozesse */
/* #(_proz_tab + 0x40) */unsigned char akt_proz_index;	/* Index auf den akt. Prozess */

};

/* Ablage der Prozess-Tabelle im XDATA */
volatile xdata at 0x1E00 struct prozess_tabelle proz_tab;






/***********************************************/
/* reiht neuen Prozess in die Prozessliste ein */
/***********************************************/
unsigned char process_start( void (*funktionszeiger)(), unsigned char zeit );

/***********************************************/
/* nimmt Prozess aus der Prozessliste heraus   */
/***********************************************/
unsigned char process_kill( unsigned char pid );


/******************************************/
/* Erweitert aktuelle Zeitscheibe auf 255 */
/******************************************/
#define process_hog() proz_tab.ticks_aktuell=255

/********************************************/
/* Veranlasst Wechsel zum naechsten Prozess */
/********************************************/
#define process_defer() { if (proz_tab.anz_proz != 1) timer2();}


/*********************************************/
/* Tickzaehler des akt. Prozesses neu setzen */
/*********************************************/
#define process_set_ticks( prozess_zeiger, prozess_ticks)  proz_tab.proz_ticks[prozess_zeiger]=prozess_ticks

/*********************************************/
/* Tickzaehler des akt. Prozesses neu setzen */
/*********************************************/
#define process_get_pid()  proz_tab.akt_proz





#endif

